;; Generated package description from page-break-lines.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "page-break-lines" "0.15.0.20260904.20" "Display ^L page breaks as tidy horizontal lines" '((emacs "25.1")) :commit "bd5eecd6580481fc37ed41c32d3c18cb49f4af57" :authors '(("Steve Purcell" . "steve@sanityinc.com")) :maintainer '("Steve Purcell" . "steve@sanityinc.com") :keywords '("convenience" "faces") :url "https://github.com/purcell/page-break-lines")
